@@ -1,3 +1,5 @@
+from typing import Optional
+
 from aiohttp import web
 from aiohttp_middlewares import cors_middleware  # type: ignore
 
@@ -47,6 +49,8 @@ async def index(request: web.Request) -> web.Response:
     host = request.headers.get("Host")
     if "serve" in host:
         redirect_to = "https://" + host
+        if request.query.get("redirect_to"):
+            redirect_to += "/" + request.query.get("redirect_to")
 
     path = {
         "tensorboard": "/tensorboard/",
@@ -95,30 +99,65 @@ async def index(request: web.Request) -> web.Response:
     return resp
 
 
+async def authorize_user_service(request: web.Request) -> web.Response:
+    unauthorized_message = (
+        "If viewing in a web browser, please first visit the FastAPI documentation link. "
+        "You can find the FastAPI link by navigating to the cluster page and clicking on "
+        "the Deployments tab. Otherwise, please include the cluster's user_service_token "
+        "in the Authorization bearer headers of the request. The user_service_token can "
+        "be found on the cluster page under Network Access."
+    )
+    if not request.config_dict["user_service_token"]:
+        return web.Response(text=unauthorized_message, status=401)
+
+    auth_token = request.config_dict["user_service_token"]
+    cookies = request.cookies
+    if cookies.get("anyscale-token") == auth_token:
+        return web.Response(text="Authorized", status=200)
+    headers = request.headers
+    if (
+        headers.get("Authorization") == f"Bearer {auth_token}"
+        or headers.get("Authorization") == f"bearer {auth_token}"
+    ):
+        return web.Response(text="Authorized", status=200)
+
+    return web.Response(text=unauthorized_message, status=401)
+
+
 async def authorize(request: web.Request) -> web.Response:
     print(
         "Got authorization request for:",
         request.headers.get("X-Forwarded-Uri", "unknown"),
     )
-    cookies = request.cookies
-    if cookies.get("anyscale-token") == request.config_dict["auth_token"]:
-        return web.Response(text="Authorized", status=200)
-    headers = request.headers
-    if (
-        headers.get("Authorization") == f"Bearer {request.config_dict['auth_token']}"
-        or headers.get("Authorization") == f"bearer {request.config_dict['auth_token']}"
-    ):
-        return web.Response(text="Authorized", status=200)
-    unauthorized_message = (
-        "Unauthorized. Please navigate to this resource through the cluster's page in the Anyscale "
-        "product. Doing so will automatically authorize this request from a web browser. Please "
-        "include the `anyscale-token` in the cookies if not performing this request from a web "
-        "browser."
-    )
-    return web.Response(text=unauthorized_message, status=401)
+
+    use_token = request.query.get("use_token", "access_token")
+    if use_token == "access_token":
+        auth_token = request.config_dict["auth_token"]
+
+        cookies = request.cookies
+        if cookies.get("anyscale-token") == auth_token:
+            return web.Response(text="Authorized", status=200)
+        headers = request.headers
+        if (
+            headers.get("Authorization") == f"Bearer {auth_token}"
+            or headers.get("Authorization") == f"bearer {auth_token}"
+        ):
+            return web.Response(text="Authorized", status=200)
+
+        unauthorized_message = (
+            "Unauthorized. Please navigate to this resource through the cluster's page in the Anyscale "
+            "product. Doing so will automatically authorize this request from a web browser. Please "
+            "include the `anyscale-token` in the cookies if not performing this request from a web "
+            "browser."
+        )
+        return web.Response(text=unauthorized_message, status=401)
+    else:
+        return await authorize_user_service(request)
 
 
-def make_auth_proxy_app(auth_token: str) -> web.Application:
+def make_auth_proxy_app(
+    auth_token: str, user_service_token: Optional[str]
+) -> web.Application:
     auth_app = web.Application()
     auth_app.add_routes(
         [
@@ -143,5 +182,6 @@ def make_auth_proxy_app(auth_token: str) -> web.Application:
 
     app.add_subapp("/auth", auth_app)
     app["auth_token"] = auth_token
+    app["user_service_token"] = user_service_token
 
     return app
