@@ -107,7 +107,7 @@ class AuthenticationBlock:
             use_asyncio (bool): If this flag is set, the client will run requests in
             a threadpool. Invocations of the api client will return coroutines.
         """
-        configuration = anyscale_client.Configuration(host=self.host + "/ext")
+        configuration = anyscale_client.Configuration(host=self.host + "/ext/v0")
         configuration.connection_pool_maxsize = 100
 
         api_client = (
