@@ -5,7 +5,9 @@ import sys
 from types import ModuleType
 from typing import Any, Dict, List, Optional, Tuple
 
-from ray.autoscaler._private.command_runner import _with_environment_variables
+from ray.autoscaler._private.command_runner import (  # pylint:disable=private-import
+    _with_environment_variables,
+)
 from ray.autoscaler.command_runner import CommandRunnerInterface
 
 from anyscale.autoscaler.docker_local import client
