@@ -6,7 +6,7 @@ AMAZON_IAM_FULL_ACCESS_POLICY_ARN = "arn:aws:iam::aws:policy/IAMFullAccess"
 AMAZON_ROUTE53_FULL_ACCESS_POLICY_ARN = (
     "arn:aws:iam::aws:policy/AmazonRoute53FullAccess"
 )
-AMAZON_S3_FULL_ACCESS_POLIC_ARN = "arn:aws:iam::aws:policy/AmazonS3FullAccess"
+AMAZON_S3_FULL_ACCESS_POLICY_ARN = "arn:aws:iam::aws:policy/AmazonS3FullAccess"
 AMAZON_VPC_FULL_ACCESS_POLICY_ARN = "arn:aws:iam::aws:policy/AmazonVPCFullAccess"
 
 EKS_FULL_ACCESS_POLICY_NAME = "EKSFullAccess"
@@ -15,6 +15,10 @@ EKS_FULL_ACCESS_POLICY_DOCUMENT = {
     "Statement": [{"Effect": "Allow", "Action": ["eks:*"], "Resource": "*"}],
 }
 
+AMAZON_ECR_READONLY_ACCESS_POLICY_ARN = (
+    "arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly"
+)
+
 ANYSCALE_IAM_ACCESS_POLICY_NAME = "AnyscaleIAMAccess"
 ANYSCALE_IAM_ACCESS_POLICY_DOCUMENT = {
     "Version": "2012-10-17",
