@@ -71,17 +71,6 @@ def cloud_set_default(
 
 
 @cloud_cli.command(name="setup", help="Set up a cloud provider.")
-@click.option(
-    "--bring-cloud-account",
-    help="Whether or not this cloud runs in a customer account.",
-    required=True,
-    prompt=(
-        "Do you want to bring your own cloud account?\n"
-        "\tSelecting 'no' means your clusters will run on Anyscale.\n"
-        "\tSelecting 'yes' means your clusters will run in your cloud account.\n"
-    ),
-    type=click.Choice(["yes", "no"], case_sensitive=False),
-)
 @click.option(
     "--provider",
     help="The cloud provider type.",
@@ -122,7 +111,6 @@ def cloud_set_default(
     "--vpc-peering-target-vpc-id", help="VPC ID of the target VPC.", required=False
 )
 def setup_cloud(
-    bring_cloud_account: str,
     provider: str,
     region: str,
     name: str,
@@ -134,7 +122,6 @@ def setup_cloud(
     vpc_peering_target_vpc_id: Optional[str],
 ) -> None:
     CloudController().setup_cloud(
-        is_aioa=bring_cloud_account == "no",
         is_k8s=not legacy,
         provider=provider,
         region=region,
