@@ -16,12 +16,9 @@ If the cluster is already running we compare the new cluster env build_id and
 compute_template_id with the new cluster, if they match we connect, if they do
 not match, we fail and require explicitly updating the cluster.
 
-Acquiring a cluster lock via the Ray client: To avoid multiple clients from
-connecting to the same cluster, by default we acquire an exclusive lock on
-the cluster. This is done by checking that "num_clients" == 1 in the
-returned connection info object, which means that we are the first client.
-You can also allow multiple clients to connect to one cluster by passing
-ANYSCALE_ALLOW_MULTIPLE_CLIENTS=1
+By default, multiple clients can connect to a cluster. If you want to
+explicitly disable multiple client connects, set
+ANYSCALE_ALLOW_MULTIPLE_CLIENTS=0 in the environment.
 """
 
 import copy
@@ -98,7 +95,9 @@ MINIMUM_RAY_VERSION = "1.4.0"
 # Default docker images to use for connect clusters.
 def _get_base_image(image: str, ray_version: str, cpu_or_gpu: str) -> str:
     py_version = "".join(str(x) for x in sys.version_info[0:2])
-    if py_version not in ["36", "37", "38", "39"]:
+    if sys.version_info.major == 3 and sys.version_info.minor == 10:
+        py_version = "310"
+    if py_version not in ["36", "37", "38", "39", "310"]:
         raise ValueError("No default docker image for py{}".format(py_version))
     return "anyscale/{}:{}-py{}-{}".format(image, ray_version, py_version, cpu_or_gpu)
 
@@ -748,17 +747,8 @@ class ClientBuilder:
             SERVE_ROOT_URL_ENV_KEY = "RAY_SERVE_ROOT_URL"
 
         try:
-            # Like "https://session-xxx.anyscaleuserdata.com"
-            base_url = cluster.host_name
-            # Like "https://serve-session-xxx.anyscaleuserdata.com". Insert "serve"
-            # after "https://" (which has length of 8)
-            serve_url = (
-                base_url[: base_url.index("https://") + 8]
-                + "serve-"
-                + base_url[base_url.index("https://") + 8 :]
-            )
             self._user_runtime_env.setdefault("env_vars", {}).setdefault(
-                SERVE_ROOT_URL_ENV_KEY, serve_url
+                SERVE_ROOT_URL_ENV_KEY, cluster.user_service_url.rstrip("/")
             )
         except Exception:
             self._log.warning(
@@ -778,17 +768,8 @@ class ClientBuilder:
         """
 
         _allow_multiple_clients = (
-            os.environ.get("ANYSCALE_ALLOW_MULTIPLE_CLIENTS") == "1"
+            os.environ.get("ANYSCALE_ALLOW_MULTIPLE_CLIENTS") != "0"
         )
-        if _allow_multiple_clients:
-            if not self._cluster_name:
-                self._log.warning(
-                    "`ANYSCALE_ALLOW_MULTIPLE_CLIENTS` only works when a cluster_name is specified"
-                )
-            else:
-                self._log.info(
-                    "`ANYSCALE_ALLOW_MULTIPLE_CLIENTS` is set, multiple clients will be allowed to connect to one cluster."
-                )
         self._log.info("Finished parsing arguments.", block_label="ParseArgs")
         self._log.close_block("ParseArgs")
 
@@ -1305,7 +1286,7 @@ class ClientBuilder:
             )
         finally:
             os.chdir(cur_dir)
-        self._os._exit(0)
+        self._os._exit(0)  # pylint:disable=private-use
 
     def _exec_self_in_local_docker(self) -> None:
         """Run the current main file in a local docker image."""
@@ -1331,7 +1312,7 @@ class ClientBuilder:
         ]
         self._log.debug("Running", command)
         self._subprocess.check_call(command)
-        self._os._exit(0)
+        self._os._exit(0)  # pylint:disable=private-use
 
 
 # This implements the following utility function for users:
