@@ -20,7 +20,7 @@ from anyscale.sdk.anyscale_client import (
     UpdateCluster,
 )
 from anyscale.sdk.anyscale_client.models.cloud import Cloud
-from anyscale.util import get_endpoint, wait_for_session_start
+from anyscale.util import generate_slug, get_endpoint, wait_for_session_start
 from anyscale.utils.connect_helpers import get_cluster, list_entities
 
 
@@ -238,8 +238,17 @@ class PrepareClusterBlock:
             The name of the cluster to connect to.
         """
         ray_cli = self._ray.util.client.ray
+        create_new_cluster = False
         if not cluster_name:
-            results = list_entities(self.anyscale_api_client.list_sessions, project_id)
+            # If a cluster name is not provided, user wants to start a new cluster.
+            create_new_cluster = True
+
+            # Try to generate an auto-incrementing cluster name from the first 100 clusters.
+            # If there are more than 40 clusters with, we will generate a random name.
+            # If the name is already taken, we will re-try with a different random name.
+            results = list_entities(
+                self.anyscale_api_client.list_sessions, project_id, max=100
+            )
             self.log.debug("-> Starting a new cluster")
             used_names = [s.name for s in results]
             for i in range(MAX_CLUSTERS):
@@ -249,15 +258,6 @@ class PrepareClusterBlock:
                     self.log.debug("Starting cluster", cluster_name)
                     break
 
-        # Should not happen.
-        if cluster_name is None:
-            raise RuntimeError(
-                "Could not create new cluster to connect to. "
-                "We automatically generate cluster names cluster-1, cluster-2, ..., "
-                f"cluster-{MAX_CLUSTERS}. Consider providing an explicit "
-                'cluster name e.g., ray.init("anyscale://cluster_name") or deleting some of these clusters.'
-            )
-
         self.log.debug(
             f"Updating {cluster_name} to use build id {build_id} and compute template id {compute_template_id}"
         )
@@ -266,26 +266,26 @@ class PrepareClusterBlock:
             self._ray.util.disconnect()
         # Update cluster.
         self.log.debug("Starting cluster with sdk and compute config.")
-        self._start_cluster(
+        return self._start_cluster(
             project_id=project_id,
             cluster_name=cluster_name,
             build_id=build_id,
             compute_template_id=compute_template_id,
             autosuspend_timeout=autosuspend_timeout,
             allow_public_internet_traffic=allow_public_internet_traffic,
+            create_new_cluster=create_new_cluster,
         )
 
-        return cluster_name
-
     def _start_cluster(
         self,
         project_id: str,
-        cluster_name: str,
+        cluster_name: Optional[str],
         build_id: str,
         compute_template_id: str,
         autosuspend_timeout: int,
         allow_public_internet_traffic: Optional[bool],
-    ) -> None:
+        create_new_cluster: bool,
+    ) -> str:
         """Create/update/start the cluster if required.
 
         Args:
@@ -306,6 +306,7 @@ class PrepareClusterBlock:
             compute_template_id,
             autosuspend_timeout,
             bool(allow_public_internet_traffic),
+            create_new_cluster=create_new_cluster,
         )
 
         url = get_endpoint(f"/projects/{project_id}/clusters/{cluster.id}")
@@ -327,15 +328,17 @@ class PrepareClusterBlock:
 
             wait_for_session_start(
                 project_id,
-                cluster_name,
+                cluster.name,
                 self.api_client,
                 log=self.log,
                 block_label=self.block_label,
             )
-            self.log.debug(f"Cluster {cluster_name} finished starting. View at {url}")
+            self.log.debug(f"Cluster {cluster.name} finished starting. View at {url}")
         else:
             self._log_cluster_configs(cluster, build_id, compute_template_id, url)
 
+        return str(cluster.name)
+
     def _log_cluster_configs(
         self, cluster: Session, build_id: str, compute_template_id: str, url: str
     ) -> None:
@@ -375,21 +378,39 @@ class PrepareClusterBlock:
 
     def _create_or_update_session_data(
         self,
-        cluster_name: str,
+        cluster_name: Optional[str],
         project_id: str,
         build_id: str,
         compute_template_id: str,
         idle_timeout: Optional[int],
         allow_public_internet_traffic: bool = False,
+        create_new_cluster: bool = False,
     ) -> Tuple[Session, bool]:
         """
+        If cluster name is None, creates a new cluster with a generates a cluster name.
+
         Creates new cluster with build_id and compute_template_id if cluster
         with `cluster_name` doesn't already exist. Otherwise, update the
         `idle_timeout` of the existing cluster if provided.
         """
 
         start_required = True
+
+        if not cluster_name:
+            # Generate a cluster name
+            cluster_name = "cluster-{}".format(generate_slug())
+
         cluster = get_cluster(self.anyscale_api_client, project_id, cluster_name)
+
+        if create_new_cluster:
+            while cluster:
+                # User wants to create a new cluster but cluster with name already exists,
+                # generate a new name until we have one that's not taken.
+                cluster_name = "cluster-{}".format(generate_slug())
+                cluster = get_cluster(
+                    self.anyscale_api_client, project_id, cluster_name
+                )
+
         if not cluster:
             # Create a new cluster if there is no existing cluster with the given cluster_name
             self.log.info(
@@ -479,7 +500,9 @@ class PrepareClusterBlock:
         Get the default cluster env build based on the local python and ray versions.
         """
         py_version = "".join(str(x) for x in sys.version_info[0:2])
-        if py_version not in ["36", "37", "38", "39"]:
+        if sys.version_info.major == 3 and sys.version_info.minor == 10:
+            py_version = "310"
+        if py_version not in ["36", "37", "38", "39", "310"]:
             raise ValueError(
                 "No default cluster env for py{}. Please use a version of python between 3.6 and 3.8.".format(
                     py_version
@@ -601,7 +624,7 @@ class PrepareClusterBlock:
                 cluster_env_name.replace(":", "-")
                 if cluster_env_name
                 else "anonymous_cluster_env-{}".format(
-                    datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+                    datetime.now().strftime("%Y-%m-%d_%H-%M-%S-%f")
                 )
             )
             self.log.info(
