@@ -15,9 +15,10 @@ from click import ClickException, INT, prompt
 from anyscale.aws_iam_policies import (
     AMAZON_CLOUDWATCH_FULL_ACCESS_POLICY_ARN,
     AMAZON_EC2_FULL_ACCESS_POLICY_ARN,
+    AMAZON_ECR_READONLY_ACCESS_POLICY_ARN,
     AMAZON_IAM_FULL_ACCESS_POLICY_ARN,
     AMAZON_ROUTE53_FULL_ACCESS_POLICY_ARN,
-    AMAZON_S3_FULL_ACCESS_POLIC_ARN,
+    AMAZON_S3_FULL_ACCESS_POLICY_ARN,
     AMAZON_VPC_FULL_ACCESS_POLICY_ARN,
     ANYSCALE_IAM_ACCESS_POLICY_DOCUMENT,
     ANYSCALE_IAM_ACCESS_POLICY_NAME,
@@ -164,7 +165,6 @@ class CloudController(BaseController):
 
     def setup_cloud(
         self,
-        is_aioa: bool,
         is_k8s: bool,
         provider: str,
         region: Optional[str],
@@ -178,10 +178,6 @@ class CloudController(BaseController):
         """
         Sets up a cloud provider
         """
-
-        if is_aioa:
-            return self.setup_aioa(provider=provider, region=region, name=name, yes=yes)
-
         if provider == "aws":
             # If the region is blank, change it to the default for AWS.
             if region is None:
@@ -242,36 +238,13 @@ class CloudController(BaseController):
                 f"Invalid Cloud provider: {provider}. Available providers are [aws, gcp]."
             )
 
-    def setup_aioa(
-        self, provider: str, region: Optional[str], name: str, yes: bool
-    ) -> None:
-        if provider == "aws":
-            if region is None:
-                region = "us-west-2"
-        else:
-            raise ClickException(
-                f"Cannot use {provider}. "
-                "Currently, only AWS is supported when running in Anyscale's cloud provider account."
-            )
-
-        self.api_client.create_cloud_api_v2_clouds_post(
-            write_cloud=WriteCloud(
-                provider=provider.upper(),
-                region=region,
-                credentials="",
-                name=name,
-                is_aioa=True,
-                is_k8s=True,  # All AIOA clouds are K8s clouds.
-            )
-        )
-
     def setup_aws(self, region: str, name: str, yes: bool = False) -> None:
         from ray.autoscaler._private.aws.config import DEFAULT_RAY_IAM_ROLE
 
         os.environ["AWS_DEFAULT_REGION"] = region
 
         confirm(
-            "\nYou are about to give anyscale full access to EC2 and IAM in your AWS account.\n\n"
+            "\nYou are about to give anyscale full access to EC2 and IAM, as well as readonly access to ECR, all in your AWS account.\n\n"
             "Continue?",
             yes,
         )
@@ -365,25 +338,9 @@ class CloudController(BaseController):
 
             role = _get_role(role_name, region)
 
+        role.attach_policy(PolicyArn=AMAZON_ECR_READONLY_ACCESS_POLICY_ARN)
         # Modified permissions from Ray (no EC2FullAccess)
-        role.attach_policy(PolicyArn=AMAZON_S3_FULL_ACCESS_POLIC_ARN)
-
-        # Also attach a role to allow it to launch more nodes with itself
-        role.Policy("PassRoleToSelf").put(
-            PolicyDocument=json.dumps(
-                {
-                    "Version": "2012-10-17",
-                    "Statement": [
-                        {
-                            "Sid": "PassRoleToSelf",
-                            "Effect": "Allow",
-                            "Action": "iam:PassRole",
-                            "Resource": role.arn,
-                        }
-                    ],
-                }
-            )
-        )
+        role.attach_policy(PolicyArn=AMAZON_S3_FULL_ACCESS_POLICY_ARN)
 
     def setup_aws_k8s_cloud(self, region: str, name: str, yes: bool) -> None:
         confirm(
