@@ -18,6 +18,7 @@ from anyscale.cluster_env import (
 )
 from anyscale.controllers.base_controller import BaseController
 from anyscale.project import (
+    get_and_validate_project_id,
     get_proj_id_from_name,
     get_project_id,
     load_project_or_throw,
@@ -39,7 +40,13 @@ from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
     CreateClusterCompute,
 )
 from anyscale.shared_anyscale_utils.util import slugify
-from anyscale.util import get_endpoint, PROJECT_NAME_ENV_VAR, wait_for_session_start
+from anyscale.shared_anyscale_utils.utils.byod import BYODInfo
+from anyscale.util import (
+    extract_versions_from_image_name,
+    get_endpoint,
+    PROJECT_NAME_ENV_VAR,
+    wait_for_session_start,
+)
 
 
 class ClusterController(BaseController):
@@ -55,12 +62,16 @@ class ClusterController(BaseController):
         cluster_name: Optional[str],
         cluster_id: Optional[str],
         cluster_env_name: Optional[str],
+        docker: Optional[str],
+        python_version: Optional[str],
+        ray_version: Optional[str],
         cluster_compute_name: Optional[str],
         cluster_compute_file: Optional[str],
         cloud_name: Optional[str],
         idle_timeout: Optional[int],
         project_id: Optional[str],
         project_name: Optional[str],
+        user_service_access: Optional[str],
     ) -> None:
         if cluster_name is not None and cluster_id is not None:
             raise click.ClickException(
@@ -79,10 +90,37 @@ class ClusterController(BaseController):
                 f"Because {cluster_compute_name or cluster_compute_file} were specified, the `--cloud_name` "
                 f"argument {cloud_name} will be ignored."
             )
+        if cluster_env_name is not None and docker is not None:
+            raise click.ClickException(
+                "`--env` and `--docker` cannot both be specified. Please only provide one"
+                "of these two arguments."
+            )
+        if docker is not None and python_version is None and ray_version is None:
+            python_version, ray_version = extract_versions_from_image_name(docker)
+
+        if docker is not None and python_version is None:
+            raise click.ClickException(
+                "`--python-version` should be specified when `--docker` is used."
+            )
+        if docker is not None and ray_version is None:
+            raise click.ClickException(
+                "`--ray-version` should be specified when `--docker` is used."
+            )
 
         project_id, cluster_name = self._get_project_id_and_cluster_name(
             cluster_id, project_id, cluster_name, project_name
         )
+
+        if docker is not None:
+            assert cluster_env_name is None
+            assert python_version
+            assert ray_version
+            cluster_env_name = BYODInfo(
+                docker_image_name=docker,
+                python_version=python_version,
+                ray_version=ray_version,
+            ).encode()
+
         cluster_env, build = self._get_cluster_env_and_build(cluster_env_name)
         cluster_compute = self._get_cluster_compute(
             cluster_compute_name, cluster_compute_file, cloud_name, project_id
@@ -102,6 +140,7 @@ class ClusterController(BaseController):
             passed_cluster_env,
             passed_cluster_compute,
             idle_timeout,
+            user_service_access,
         )
 
         url = get_endpoint(f"/projects/{project_id}/clusters/{cluster.id}")
@@ -192,7 +231,7 @@ class ClusterController(BaseController):
         if cluster_id is not None:
             # Get cluster from id if specified
             cluster_list.append(
-                self.api_client.get_decorated_sessions_api_v2_decorated_sessions_session_id_get(
+                self.api_client.get_decorated_cluster_api_v2_decorated_sessions_cluster_id_get(
                     cluster_id
                 ).result
             )
@@ -202,7 +241,7 @@ class ClusterController(BaseController):
                 cluster_id, project_id, cluster_name, project_name
             )
             cluster_list.extend(
-                self.api_client.list_decorated_sessions_api_v2_decorated_sessions_get(
+                self.api_client.list_decorated_clusters_api_v2_decorated_sessions_get(
                     project_id=project_id, name_match=cluster_name
                 ).results
             )
@@ -224,14 +263,14 @@ class ClusterController(BaseController):
                         )
                         project_id = default_project.id
                         self.log.info(
-                            f"No project context detected or `--project-id` provided. Using default project {project_id}"
+                            "No project context detected or `--project-id` provided. Continuing without a project."
                         )
                 search_clusters_query["project_id"] = project_id
             if not include_inactive:
                 search_clusters_query["state_filter"] = ["Running"]
 
             # Page through all clusters in response
-            cluster_list_resp = self.api_client.list_decorated_sessions_api_v2_decorated_sessions_get(
+            cluster_list_resp = self.api_client.list_decorated_clusters_api_v2_decorated_sessions_get(
                 **search_clusters_query, count=20
             )
             next_paging_token = cluster_list_resp.metadata.next_paging_token
@@ -240,7 +279,7 @@ class ClusterController(BaseController):
                 len(cluster_list) < max_items
             )
             while has_more:
-                cluster_list_resp = self.api_client.list_decorated_sessions_api_v2_decorated_sessions_get(
+                cluster_list_resp = self.api_client.list_decorated_clusters_api_v2_decorated_sessions_get(
                     **search_clusters_query, paging_token=next_paging_token, count=20
                 )
                 next_paging_token = cluster_list_resp.metadata.next_paging_token
@@ -299,34 +338,12 @@ class ClusterController(BaseController):
             except Exception:
                 raise click.ClickException(f"No cluster exists with id {cluster_id}.")
         else:
-            if project_id:
-                validate_project_id(project_id, self.api_client)
-            elif project_name:
-                project_id = get_proj_id_from_name(project_name, self.api_client)
-            else:
-                try:
-                    # .anyscale.yaml detected in root directory and has correct format
-                    project_definition = load_project_or_throw()
-                    project_id = get_project_id(project_definition.root)
-                    project_name = self.anyscale_api_client.get_project(
-                        project_id
-                    ).result.name
-                    self.log.warning(
-                        ".anyscale.yaml has been detected in this project directory. The functionality of "
-                        "using this file to select the project has been deprecated, and will be removed "
-                        f"in April 2022. Please instead specify `--project-id {project_id}` in your command. "
-                        "The project name can also be specified by setting the environment variable "
-                        f'`{PROJECT_NAME_ENV_VAR}="{project_name}"`.'
-                    )
-                except click.ClickException:
-                    default_project = (
-                        self.anyscale_api_client.get_default_project().result
-                    )
-                    project_id = default_project.id
-                    self.log.info(
-                        f"No project context detected or `--project-id` provided. Using default project {project_id}"
-                    )
-            assert project_id  # For mypy
+            project_id = get_and_validate_project_id(
+                project_id=project_id,
+                project_name=None,
+                api_client=self.api_client,
+                anyscale_api_client=self.anyscale_api_client,
+            )
             cluster_name = self._get_or_generate_cluster_name(project_id, cluster_name)
         return project_id, cluster_name
 
@@ -398,6 +415,7 @@ class ClusterController(BaseController):
         passed_cluster_env: bool,
         passed_cluster_compute: bool,
         idle_timeout: Optional[int],
+        user_service_access: Optional[str],
     ) -> Tuple[Cluster, bool]:
         """
         Creates new cluster with specified parameters if it doesn't already exist. Otherwise
@@ -418,6 +436,8 @@ class ClusterController(BaseController):
                 "cluster_compute_id": cluster_compute.id,
                 "cluster_environment_build_id": build.id,
             }
+            if user_service_access:
+                create_cluster_data["user_service_access"] = user_service_access
             if idle_timeout:
                 create_cluster_data["idle_timeout_minutes"] = idle_timeout
 
