@@ -1,11 +1,15 @@
 from contextlib import contextmanager
 import os
+import subprocess
 import time
 from typing import Any, Dict, Optional, Tuple
 
 import click
 
-from anyscale.background.job_runner import _run_kill_child
+# TODO(tchordia) get rid of this once the local run_kill_child is in prod
+from anyscale.background.job_runner import (  # pylint:disable=private-import
+    _run_kill_child,
+)
 from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client.models.app_config_config_schema import (
     AppConfigConfigSchema,
@@ -368,3 +372,51 @@ class ConfigController(BaseController):
                 raise click.ClickException(message) from e
             else:
                 self.log.warning(message)
+
+
+# TODO(tchordia) once this function is in prod, remove last dependency from anyscale.background
+def run_kill_child(
+    *popenargs, input=None, timeout=None, check=False, **kwargs
+) -> subprocess.CompletedProcess:
+    """
+    This function is a fork of subprocess.run with fewer args.
+    The goal is to create a child subprocess that is GUARANTEED to exit when the parent exits
+    This is accomplished by:
+    1. Making sure the child is the head of a new process group
+    2. Create a third "Killer" process that is responsible for killing the child when the parent dies
+    3. Killer process checks every second if the parent is dead.
+    4. Killing the entire process group when we want to kill the child
+
+    Arguments are the same as subprocess.run
+    """
+    # Start new session ensures that this subprocess starts as a new process group
+    with subprocess.Popen(start_new_session=True, *popenargs, **kwargs) as process:
+        parent_pid = os.getpid()
+        child_pid = process.pid
+        child_pgid = os.getpgid(child_pid)
+
+        # Open a new subprocess to kill the child process when the parent process dies
+        # kill -s 0 parent_pid will succeed if the parent is alive.
+        # If it fails, SIGKILL the child process group and exit
+        subprocess.Popen(
+            f"while kill -s 0 {parent_pid}; do sleep 1; done; kill -9 -{child_pgid}",
+            shell=True,
+            # Suppress output
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+        )
+
+        try:
+            stdout, stderr = process.communicate(input, timeout=timeout)
+        except:  # noqa      (this is taken from subprocess.run directly)
+            # Including KeyboardInterrupt, communicate handled that.
+            process.kill()
+            # We don't call process.wait() as .__exit__ does that for us.
+            raise
+
+        retcode = process.poll()
+        if check and retcode:
+            raise subprocess.CalledProcessError(
+                retcode, process.args, output=stdout, stderr=stderr
+            )
+    return subprocess.CompletedProcess(process.args, retcode or 0, stdout, stderr)
