@@ -4,6 +4,7 @@ import os
 from pathlib import Path
 import time
 from typing import Any, cast, Dict, List, Optional, Tuple, Type, Union
+from urllib.parse import urlparse
 
 import click
 from pydantic import BaseModel, Field, root_validator
@@ -13,7 +14,7 @@ import tabulate
 from urllib3 import Retry
 import yaml
 
-from anyscale.cli_logger import BlockLogger
+from anyscale.cli_logger import LogsLogger
 from anyscale.client.openapi_client import (
     CreateProductionJob,
     DecoratedProductionJob,
@@ -37,10 +38,35 @@ from anyscale.project import (
     get_project_id,
     ProjectDefinition,
 )
-from anyscale.util import get_endpoint, PROJECT_NAME_ENV_VAR
+from anyscale.shared_anyscale_utils.utils.byod import BYODInfo
+from anyscale.util import (
+    extract_versions_from_image_name,
+    get_endpoint,
+    poll,
+    PROJECT_NAME_ENV_VAR,
+)
+from anyscale.utils.runtime_env import upload_and_rewrite_working_dir
 
 
-log = BlockLogger()
+log = LogsLogger()
+
+terminal_state = {
+    HaJobStates.SUCCESS,
+    HaJobStates.TERMINATED,
+    HaJobStates.BROKEN,
+    HaJobStates.OUT_OF_RETRIES,
+}
+pending_states = {
+    HaJobStates.PENDING,
+    HaJobStates.AWAITING_CLUSTER_START,
+    HaJobStates.ERRORED,
+}
+
+
+class MiniJobRun(BaseModel):
+    last_job_run_id: str
+    job_state: str
+    error: Optional[str]
 
 
 def _validate_conda_option(conda_option: Union[str, Dict]) -> Union[str, Dict]:
@@ -111,16 +137,70 @@ def _validate_py_modules(py_modules_option: List[str]) -> List[str]:
     return py_modules_option
 
 
+def _working_dir_is_remote_uri(working_dir: str) -> bool:
+    return "://" in working_dir
+
+
 def _validate_working_dir(working_dir_option: str) -> str:
-    if "://" not in working_dir_option:
-        raise click.ClickException(
-            "Only remote URIs are currently supported for working_dir in the job "
-            "config (not local directories). Please see "
-            "https://docs.ray.io/en/master/handling-dependencies.html#remote-uris for supported options."
-        )
+    """If working_dir is a local directory, check that it exists."""
+    if not _working_dir_is_remote_uri(working_dir_option):
+        # We have been given a path to a local directory.
+        if not Path(working_dir_option).is_dir():
+            raise click.ClickException(
+                f"working_dir {working_dir_option} is not a valid local directory or remote URI."
+            )
+
     return working_dir_option
 
 
+def _validate_working_dir_and_upload_path(
+    working_dir: Optional[str], upload_path: Optional[str]
+):
+    """Check that the combination of working_dir and upload_path is valid.
+
+    The rule is that working_dir is a local directory if and only if upload_path is specified.
+    """
+    if upload_path is not None:
+        if working_dir is None or _working_dir_is_remote_uri(working_dir):
+            raise click.ClickException(
+                f"`upload_path` was specified, but `working_dir` is not a local directory.  Received `upload_path`: {upload_path} and `working_dir`: {working_dir}."
+            )
+    else:
+        if working_dir is not None and not _working_dir_is_remote_uri(working_dir):
+            raise click.ClickException(
+                f"{working_dir} is a local directory, but `upload_path` was not specified in the runtime_env. Please specify a remote URI in `upload_path`, e.g. `s3://my_bucket/my_dir`."
+            )
+
+
+def _validate_upload_path(upload_path: str) -> str:
+    """Check that the upload path is a valid S3 or GS remote URI."""
+    try:
+        parsed_upload_path = urlparse(upload_path)
+    except Exception as e:
+        raise click.ClickException(
+            f"Failed to parse `upload_path` {upload_path} as a URI (e.g. 's3://my_bucket/my_dir'): {e}"
+        )
+    if parsed_upload_path.scheme not in ["s3", "gs"]:
+        raise click.ClickException(
+            f"Only Amazon S3 (e.g. 's3://bucket', 's3://bucket/path') and Google Storage URIs (e.g. 'gs://bucket', 'gs://bucket/path') are supported. Received {upload_path}."
+        )
+    return upload_path
+
+
+def _validate_env_vars(env_vars: Dict[str, str]) -> Dict[str, str]:
+    for key, value in env_vars.items():
+        if not isinstance(key, str):
+            raise click.ClickException(
+                f"env_vars key {key} is not a string. Please check the formatting."
+            )
+        if not isinstance(value, str):
+            raise click.ClickException(
+                f"env_vars value {value} is not a string. Please check the formatting."
+            )
+
+    return env_vars
+
+
 class JobConfig(BaseModel):
     """
     Job Config model for CLI. Validate and preprocess so `entrypoint`, `runtime_env_config`,
@@ -152,6 +232,13 @@ class JobConfig(BaseModel):
         None,
         description="The name of the cluster environment and build revision in format `my_cluster_env:1`.",
     )
+    docker: Optional[str] = Field(None, description="Custom docker image name.")
+    python_version: Optional[str] = Field(
+        None, description="Python version for the custom docker image."
+    )
+    ray_version: Optional[str] = Field(
+        None, description="Ray version for the custom docker image."
+    )
     compute_config_id: Optional[str] = Field(
         None,
         description="The id of the compute configuration that you want to use. This id will specify the resources required for your job",
@@ -195,6 +282,44 @@ class JobConfig(BaseModel):
             values["project_id"] = get_proj_id_from_name(project_name)
         return values
 
+    @root_validator
+    def fill_cluster_env_from_custom_docker(cls: Any, values: Any) -> Any:  # noqa: N805
+        docker, python_version, ray_version, cluster_env, build_id = (
+            values.get("docker"),
+            values.get("python_version"),
+            values.get("ray_version"),
+            values.get("cluster_env"),
+            values.get("build_id"),
+        )
+        if docker is not None:
+            if cluster_env is not None:
+                raise click.ClickException(
+                    "`cluster_env` and `docker` cannot both be specified. Please only provide one"
+                    "of these in the job config file."
+                )
+            if build_id is not None:
+                raise click.ClickException(
+                    "`build_id` and `docker` cannot both be specified. Please only provide one"
+                    "of these in the job config file."
+                )
+            if python_version is None and ray_version is None:
+                python_version, ray_version = extract_versions_from_image_name(docker)
+
+            if python_version is None:
+                raise click.ClickException(
+                    "`python_version` should be specified when `docker` is used."
+                )
+            if ray_version is None:
+                raise click.ClickException(
+                    "`ray_version` should be specified when `docker` is used."
+                )
+            values["cluster_env"] = BYODInfo(
+                docker_image_name=docker,
+                python_version=python_version,
+                ray_version=ray_version,
+            ).encode()
+        return values
+
     @root_validator
     def fill_build_id(cls: Any, values: Any) -> Any:  # noqa: N805
         build_id, cluster_env = (
@@ -273,6 +398,13 @@ class JobConfig(BaseModel):
                         f"runtime_env['py_modules'] must be list, got type({py_modules_option})."
                     )
                 runtime_env["py_modules"] = _validate_py_modules(py_modules_option)
+            if "upload_path" in runtime_env:
+                upload_path_option = runtime_env["upload_path"]
+                if not isinstance(upload_path_option, str):
+                    raise click.ClickException(
+                        f"runtime_env['upload_path'] must be str, got type({upload_path_option})."
+                    )
+                runtime_env["upload_path"] = _validate_upload_path(upload_path_option)
             if "working_dir" in runtime_env:
                 working_dir_option = runtime_env["working_dir"]
                 if not isinstance(working_dir_option, str):
@@ -280,6 +412,16 @@ class JobConfig(BaseModel):
                         f"runtime_env['working_dir'] must be str, got type({working_dir_option})."
                     )
                 runtime_env["working_dir"] = _validate_working_dir(working_dir_option)
+            _validate_working_dir_and_upload_path(
+                runtime_env.get("working_dir"), runtime_env.get("upload_path")
+            )
+            if "env_vars" in runtime_env:
+                env_vars_option = runtime_env["env_vars"]
+                if not isinstance(env_vars_option, dict):
+                    raise click.ClickException(
+                        f"runtime_env['env_vars'] must be dict, got type({env_vars_option})."
+                    )
+                runtime_env["env_vars"] = _validate_env_vars(env_vars_option)
             values["runtime_env"] = runtime_env
 
         return values
@@ -296,12 +438,22 @@ class LogProvider(ABC):
     ) -> List[Tuple[int, str]]:
         """Query logs for a time range, return list of (timestamp_ns, log_line)."""
 
+    @abstractmethod
+    def close(self) -> None:
+        """Close any connection this log provider uses"""
+
+    def __enter__(self, *args, **kwargs):
+        pass
+
+    def __exit__(self, *args, **kwargs):
+        self.close()
+
 
 class LokiLogProvider(LogProvider):
     def __init__(self, dns_name: str, anyscale_token: str, query: str) -> None:
         # https://grafana.com/docs/loki/latest/api/#get-lokiapiv1query_range
         self.url = f"https://{dns_name}/loki/api/v1/query_range"
-        self.params = {
+        self.params: Dict[str, Union[str, int]] = {
             "token": anyscale_token,
             "query": query,
             "limit": 1000,
@@ -310,6 +462,7 @@ class LokiLogProvider(LogProvider):
             "direction": "forward",
         }
 
+    def __enter__(self, *args, **kwargs):
         self.session = requests.Session()
         self.session.mount(
             "https://",
@@ -317,6 +470,13 @@ class LokiLogProvider(LogProvider):
                 max_retries=Retry(total=10, backoff_factor=0.1, allowed_methods=["GET"])
             ),
         )
+        return self
+
+    def close(self) -> None:
+        self.session.close()
+
+    def __exit__(self, *args, **kwargs):
+        self.close()
 
     def query(
         self, start_timestamp_ns: int, end_timestamp_ns: int
@@ -344,7 +504,9 @@ class LokiLogProvider(LogProvider):
 
 class JobController(BaseController):
     def __init__(
-        self, log: BlockLogger = BlockLogger(), initialize_auth_api_client: bool = True
+        self,
+        log: LogsLogger = LogsLogger(),  # The default logger is shared across instances of this class.
+        initialize_auth_api_client: bool = True,
     ):
         super().__init__(initialize_auth_api_client=initialize_auth_api_client)
         self.log = log
@@ -352,7 +514,7 @@ class JobController(BaseController):
 
     def submit(
         self, job_config_file: str, name: Optional[str], description: Optional[str]
-    ) -> None:
+    ) -> str:
         job_config = self._read_job_config(job_config_file)
         project_id = job_config.project_id
 
@@ -363,20 +525,14 @@ class JobController(BaseController):
             if root_dir is not None:
                 project_definition = ProjectDefinition(root_dir)
                 project_id = get_project_id(project_definition.root)
-                project_name = self.anyscale_api_client.get_project(
-                    project_id
-                ).result.name
-                self.log.warning(
-                    ".anyscale.yaml has been detected in this project directory. The functionality of "
-                    "using this file to select the project has been deprecated, and will be removed "
-                    f"in April 2022. Please instead specify `project_id: {project_id}` in the job config file. "
-                    "The project name can also be specified by setting the environment variable "
-                    f'`{PROJECT_NAME_ENV_VAR}="{project_name}"`.'
-                )
             else:
                 default_project = self.anyscale_api_client.get_default_project().result
                 project_id = default_project.id
                 self.log.info("No project specified. Continuing without a project.")
+        if job_config.runtime_env:
+            job_config.runtime_env = upload_and_rewrite_working_dir(
+                job_config.runtime_env, self.log
+            )
 
         config_object = ProductionJobConfig(
             entrypoint=job_config.entrypoint,
@@ -409,6 +565,7 @@ class JobController(BaseController):
             f"Get the logs for the job with `anyscale job logs --job-id {job.id} --follow`."
         )
         self.log.info(f'View the job in the UI at {get_endpoint(f"/jobs/{job.id}")}.')
+        return job.id
 
     def _read_job_config(self, job_config_file: str) -> JobConfig:
         if not os.path.exists(job_config_file):
@@ -434,6 +591,9 @@ class JobController(BaseController):
         Both functionalities are combined in one function because the API to list both these
         entities is currently the same.
         """
+        entity_type = "services" if is_service else "jobs"
+        print(f'View your {entity_type} in the UI at {get_endpoint(f"/{entity_type}")}')
+
         jobs_list = []
         if job_id:
             job = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
@@ -451,7 +611,30 @@ class JobController(BaseController):
                     f"ID {job_id} belongs to a Anyscale job. Please get information about "
                     f"this job with `anyscale job list --job-id {job_id}`."
                 )
-            jobs_list.append(job)
+            output_map = {
+                "Name": job.name,
+                "Id": job.id,
+                "Cost (dollars)": job.cost_dollars,
+                "Project name": job.project.name,
+                "Cluster name": job.last_job_run.cluster.name
+                if job.last_job_run and job.last_job_run.cluster
+                else None,
+                "Current state": job.state.current_state,
+                "Creator": job.creator.username,
+                "Entrypoint": job.config.entrypoint
+                if len(job.config.entrypoint) < 100
+                else job.config.entrypoint[:100] + " ...",
+            }
+            if is_service:
+                output_map["Access"] = job.access
+                output_map["URL"] = job.url
+                output_map["Token"] = job.token
+
+            output_str = "\n".join(
+                [f"\t{key}: {output_map[key]}" for key in output_map]
+            )
+            print(output_str)
+            return
         else:
             if not include_all_users:
                 creator_id = (
@@ -515,8 +698,6 @@ class JobController(BaseController):
             ],
             tablefmt="plain",
         )
-        entity_type = "services" if is_service else "jobs"
-        print(f'View your {entity_type} in the UI at {get_endpoint(f"/{entity_type}")}')
         print(f"{entity_type.capitalize()}:\n{table}")
 
     def terminate(
@@ -570,70 +751,139 @@ class JobController(BaseController):
         ).result
         return jobs_resp
 
+    def _get_latest_job_run(self, job_id: str) -> Any:
+        job_object = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
+            production_job_id=job_id
+        ).result
+        return job_object
+
+    def _get_formatted_latest_job_run(self, job_id: str) -> Optional[MiniJobRun]:
+        job_object = self._get_latest_job_run(job_id)
+        job_state = job_object.state.current_state
+        last_job_run_id = job_object.last_job_run_id
+        if job_state in terminal_state and last_job_run_id is None:
+            raise click.ClickException(
+                f"Can't find latest job run for {job_state} job."
+            )
+        if not last_job_run_id:
+            return None
+        return MiniJobRun(
+            last_job_run_id=last_job_run_id,
+            job_state=job_state,
+            error=job_object.state.error,
+        )
+
     def _wait_for_a_job_run(self, job_id: str) -> str:
         """Given job_id, wait until there is a job run on a cluster."""
         last_job_run_id = None
-        terminal_state = {
-            HaJobStates.SUCCESS,
-            HaJobStates.TERMINATED,
-            HaJobStates.BROKEN,
-            HaJobStates.OUT_OF_RETRIES,
-        }
-        while last_job_run_id is None:
-            job_object = self.api_client.get_job_api_v2_decorated_ha_jobs_production_job_id_get(
-                production_job_id=job_id
-            ).result
-            job_state = job_object.state.current_state
-            last_job_run_id = job_object.last_job_run_id
-            if job_state in terminal_state and last_job_run_id is None:
-                raise click.ClickException(
-                    f"Can't find latest job run for {job_state} job."
-                )
-            if last_job_run_id is None:
-                self.log.info(f"Waiting for a job run, current state is {job_state}.")
-                time.sleep(5)
+        with self.log.spinner("Waiting for job run...") as spinner:
+            while last_job_run_id is None:
+                job_object = self._get_latest_job_run(job_id)
+                job_state = job_object.state.current_state
+                last_job_run_id = job_object.last_job_run_id
+                if job_state in terminal_state and last_job_run_id is None:
+                    raise click.ClickException(
+                        f"Can't find latest job run for {job_state} job."
+                    )
+                if last_job_run_id is None:
+                    spinner.text = (
+                        f"Waiting for a job run, current state is {job_state}..."
+                    )
+                    time.sleep(5)
         return last_job_run_id
 
+    def _wait_for_job_running(self, job_id: str) -> None:
+        with self.log.spinner("Waiting for job to retry...") as spinner:
+            for _ in poll():
+                job_object = self._get_latest_job_run(job_id)
+                job_state = job_object.state.current_state
+                if job_state in pending_states:
+                    spinner.text = (
+                        f"Waiting for job to retry, current state is {job_state}..."
+                    )
+                else:
+                    return
+
     def logs(
         self,
-        job_id: Optional[str],
-        job_name: Optional[str],
+        job_id: Optional[str] = None,
+        job_name: Optional[str] = None,
         should_follow: bool = False,
         log_provider_cls: Type[LogProvider] = LokiLogProvider,
     ) -> None:
-        job_id = self._resolve_job_object(job_id, job_name).id
-        self._wait_for_a_job_run(cast(str, job_id))
-
-        # TODO(simon): implement ability to look through logs from previous tries
-        log_info: JobsLogsQueryInfo = self.api_client.get_job_logs_query_info_api_v2_decorated_ha_jobs_production_job_id_logs_query_get(
-            production_job_id=job_id
-        ).result
-
-        log_provider = log_provider_cls(
-            dns_name=log_info.loki_dns_name,
-            anyscale_token=log_info.access_token,
-            query=log_info.loki_query,
-        )
+        job_id = cast(str, self._resolve_job_object(job_id, job_name).id)
+        self._wait_for_a_job_run(job_id)
 
         start_timestamp_ns = 0
-        end_timestamp_ns = int(time.time() * 1e9)
-        while True:
-            lines = log_provider.query(start_timestamp_ns, end_timestamp_ns)
+        max_iter = 1 if not should_follow else None
+        log_provider_map: Dict[str, LogProvider] = {}
 
-            for time_stamp, line in lines:
-                # Update the start timestamp in case we are following or paginated.
-                start_timestamp_ns = time_stamp
-                print(line)
+        for _ in poll(1, max_iter=max_iter):
+            # Fetch the latest job run
+            jr = self._get_formatted_latest_job_run(job_id)
 
-            # No log entry for our time range.
-            if len(lines) == 0:
-                if should_follow:
-                    # Avoid repeatedly polling empty log entry. Sleep here to give loki server a break.
-                    time.sleep(1)
-                else:
-                    break
+            # This shouldn't happen because we are waiting for it above
+            assert jr, "Could not find the latest job run"
 
-            # Update the params for updated range.
-            start_timestamp_ns += 1  # Start time should exclude the previous entries.
-            if should_follow:  # end time should only be updated when we are following.
-                end_timestamp_ns = int(time.time() * 1e9)
+            log_provider = log_provider_map.get(jr.last_job_run_id)
+            if not log_provider:
+                # Open a new CLI block
+                # This is to ensure the logs for each job are nicely formatted
+                self.log.open_block(
+                    jr.last_job_run_id,
+                    f"Job Run Id: {jr.last_job_run_id}",
+                    auto_close=True,
+                )
+
+                # We have not seen this job run id before. We need to construct a new log provider
+                log_info: JobsLogsQueryInfo = self.api_client.get_job_logs_query_info_api_v2_decorated_ha_jobs_production_job_id_logs_query_get(
+                    production_job_id=job_id
+                ).result
+
+                log_provider = log_provider_cls(
+                    dns_name=log_info.loki_dns_name,
+                    anyscale_token=log_info.access_token,
+                    query=log_info.loki_query,
+                )
+
+                # Add to the cache
+                log_provider_map[jr.last_job_run_id] = log_provider
+
+            # Make sure we only have 1 open request session at a time
+            with log_provider:
+
+                # Flush all logs in the loki endpoint
+                last_received_logs_iteration = 0
+                for iteration in poll(interval_secs=1):
+                    # Fetch logs in the time range
+                    end_timestamp_ns = int(time.time() * 1e9)
+                    lines = log_provider.query(start_timestamp_ns, end_timestamp_ns)
+
+                    # Print the results
+                    for time_stamp, line in lines:
+                        # Update the start timestamp in case we are following or paginated.
+                        start_timestamp_ns = time_stamp + 1
+                        self.log.log(line)
+                    if len(lines) > 0:
+                        # Record the last iteration at which we received logs
+                        last_received_logs_iteration = iteration
+
+                    # If we are following, threshold is 30 iterations (or 30 seconds)
+                    threshold = 30 if should_follow else 0
+                    if iteration - last_received_logs_iteration > threshold:
+                        # If we haven't received logs after threshold iterations, assume existing logs have been flushed
+                        break
+
+                # We have already flushed existing logs for this job run
+                # If the job is in a terminal state, exit
+                if jr.job_state in terminal_state:
+                    # If the state is terminal, exit
+                    self.log.info(f"Job entered terminal state {jr.job_state}.")
+                    if jr.error:
+                        self.log.error(f"Job exited with error: {jr.error}")
+                    return
+
+                # If the job is in a pending state, wait for it to start running
+                # before fetching more logs
+                if jr.job_state in pending_states:
+                    self._wait_for_job_running(job_id)
