@@ -42,11 +42,6 @@ class ProjectController(BaseController):
         self.log.open_block("Output")
 
     def clone(self, project_name: str, owner: Optional[str] = None) -> None:
-        message = (
-            "Warning: `anyscale clone` has been deprecated and will be removed in the future. "
-            "Please use `anyscale project init --project-id [project_id]` to connect to an existing project."
-        )
-        self.log.warning(message)
         project_id = get_proj_id_from_name(project_name, self.api_client, owner)
 
         os.makedirs(project_name)
