@@ -11,7 +11,7 @@ from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import Project
 from anyscale.client.openapi_client.api.default_api import DefaultApi
 from anyscale.shared_anyscale_utils.util import slugify
-from anyscale.util import get_endpoint, send_json_request
+from anyscale.util import get_endpoint, PROJECT_NAME_ENV_VAR, send_json_request
 
 
 ANYSCALE_PROJECT_FILE = ".anyscale.yaml"
@@ -331,3 +331,29 @@ def validate_project_id(project_id, api_client: DefaultApi) -> Project:
         )
 
     return resp.result
+
+
+def get_and_validate_project_id(
+    project_id: Optional[str],
+    project_name: Optional[str],
+    api_client: DefaultApi,
+    anyscale_api_client: DefaultApi,
+) -> str:
+    project_name = project_name or os.environ.get(PROJECT_NAME_ENV_VAR)
+    if project_id:
+        validate_project_id(project_id, api_client)
+    elif project_name:
+        project_id = get_proj_id_from_name(project_name, api_client)
+    else:
+        try:
+            # .anyscale.yaml detected in root directory and has correct format
+            project_definition = load_project_or_throw()
+            project_id = get_project_id(project_definition.root)
+        except click.ClickException:
+            default_project = anyscale_api_client.get_default_project().result
+            project_id = default_project.id
+            log.info(
+                "No project context detected or `--project-id` provided. Continuing without a project."
+            )
+    assert project_id  # for mypy
+    return project_id
