@@ -32,6 +32,7 @@ from anyscale.commands.session_commands import (
     anyscale_up,
 )
 from anyscale.commands.session_commands_hidden import session_cli
+from anyscale.commands.workspace_commands import workspace_cli
 import anyscale.conf
 from anyscale.util import init_sentry
 
@@ -122,6 +123,7 @@ cli.add_command(cluster_env_cli)
 cli.add_command(job_cli)
 cli.add_command(service_cli)
 cli.add_command(cluster_cli)
+cli.add_command(workspace_cli)
 
 cli.add_command(anyscale_init)
 cli.add_command(anyscale_up)
