@@ -35,6 +35,8 @@ logger = logging.getLogger(__file__)
 BOTO_MAX_RETRIES = 5
 PROJECT_NAME_ENV_VAR = "ANYSCALE_PROJECT_NAME"
 
+VALID_PYTHON_VERSIONS = ["py36", "py37", "py38", "py39"]
+
 log = BlockLogger()  # Anyscale CLI Logger
 
 # Cached 07-21-2021
@@ -72,7 +74,7 @@ def send_json_request_raw(
     get_auth_api_client(cli_token=cli_token, host=host)
 
     url = get_endpoint(endpoint, host=host)
-    cookies = {"cli_token": cli_token or anyscale.conf.CLI_TOKEN}
+    cookies = {"cli_token": cli_token or anyscale.conf.CLI_TOKEN or ""}
     try:
         if method == "GET":
             resp = requests.get(url, params=json_args, cookies=cookies)
@@ -289,13 +291,13 @@ class Timer:
 
 def check_is_feature_flag_on(flag_key: str, default: bool = False) -> bool:
     try:
-        use_snapshot_resp = send_json_request(
+        is_on = send_json_request(
             "/api/v2/userinfo/check_is_feature_flag_on", {"flag_key": flag_key},
         )
     except Exception:
         return default
 
-    return cast(bool, use_snapshot_resp["result"]["is_on"])
+    return cast(bool, is_on["result"]["is_on"])
 
 
 def get_active_sessions(
@@ -652,7 +654,7 @@ def populate_session_args(cluster_config_str: str, config_file_name: str) -> str
 
     env = jinja2.Environment()
     t = env.parse(cluster_config_str)
-    for elem in t.body[0].nodes:
+    for elem in t.body[0].nodes:  # type: ignore
 
         if isinstance(elem, jinja2.nodes.Getattr):  # type: ignore
             if elem.attr not in os.environ:
@@ -820,3 +822,84 @@ def _update_external_ids_for_policy(
         for i in range(len(policy.get("Statement", [])))
     ]
     return policy
+
+
+def extract_versions_from_image_name(image_name: str) -> Tuple[str, str]:
+    """Returns the python version and ray extracted from an image tag.
+
+    Args:
+        image_name: e.g. anyscale/ray-ml:1.11.1-py38-gpu
+
+    Returns:
+        The (python version, ray version), e.g. ("py38", "1.11.1")
+    """
+    # e.g. 1.11.1-py32-gpu
+    image_version = image_name.split(":")[-1]
+    parts = image_version.split("-")
+    # e.g. 1.11.1
+
+    if len(parts) < 2:
+        raise ValueError(
+            f"Expected the docker image name have an image version tag (something like ray-ml:1.11.1-py38-gpu), got {image_version}."
+        )
+
+    ray_version = parts[0]
+    # Verify ray_version is valid.
+    _ray_version_major_minor(ray_version)
+
+    python_version = parts[1]
+    _check_python_version(python_version)
+    return (python_version, ray_version)
+
+
+def _ray_version_major_minor(ray_version: str) -> Tuple[int, int]:
+    """Takes in a Ray version such as "1.9.0rc1", "1.10.2", "2.0.0dev0".
+
+    Returns the major minor pair e.g. (1,9) (1,10) (2,0).
+
+    To avoid introducing undesirable dependencies, partly duplicates logic from the Anyscale
+    backend.
+    """
+    invalid_ray_version_msg = (
+        f"The Ray version `{ray_version}` has an unexpected format."
+    )
+    version_components = ray_version.split(".")
+    assert len(version_components) >= 2, invalid_ray_version_msg
+    major_str, minor_str = version_components[:2]
+    assert major_str.isnumeric() and minor_str.isnumeric(), invalid_ray_version_msg
+    major_int, minor_int = int(major_str), int(minor_str)
+    return (major_int, minor_int)
+
+
+def _check_python_version(python_version: str) -> None:
+    assert (
+        python_version in VALID_PYTHON_VERSIONS
+    ), f"Expected python_version to be one of {VALID_PYTHON_VERSIONS}, got {python_version}."
+
+
+def sleep_till(wake_time: float) -> None:
+    """ Sleep till the designated time
+    """
+    time.sleep(max(0, wake_time - time.time()))
+
+
+def poll(
+    interval_secs: float = 1,
+    timeout_secs: Optional[float] = None,
+    max_iter: Optional[int] = None,
+) -> Generator[int, None, None]:
+    """ Poll every interval_secs, until timeout_secs, or max iterations has been reached.
+    Yield the iteration number, starting at 1.
+    """
+    last_poll_time = time.time()
+    end_time = time.time() + timeout_secs if timeout_secs else None
+    count = 0
+    should_continue_iter = max_iter is None or count < max_iter
+    should_continue_time = end_time is None or time.time() < end_time
+    while should_continue_iter and should_continue_time:
+        count += 1
+        last_poll_time = time.time()
+        yield count
+        sleep_till(last_poll_time + interval_secs)
+        should_continue_iter = max_iter is None or count < max_iter
+        should_continue_time = end_time is None or time.time() < end_time
