@@ -14,21 +14,28 @@ import shutil
 import sys
 import tarfile
 import tempfile
-from typing import Any, Dict, List, Optional, Tuple, TYPE_CHECKING
+from typing import Any, Callable, Dict, List, Optional, Tuple, TYPE_CHECKING
 from urllib.parse import urlparse
 import uuid
 
-import aiobotocore  # type: ignore
+import aiobotocore
+import click
 import conda_pack  # type: ignore
 from filelock import FileLock
 import yaml
 
 from anyscale.api import configure_open_api_client_headers
+from anyscale.cli_logger import BlockLogger  # type: ignore
 import anyscale.utils.ray_conda as ray_conda
-from anyscale.utils.ray_utils import _dir_travel, _get_excludes  # type: ignore
+from anyscale.utils.ray_utils import (  # type: ignore
+    _dir_travel,
+    _get_excludes,
+    zip_directory,
+)
 
 
 if TYPE_CHECKING:
+    from aiobotocore import AioSession
     from ray.job_config import JobConfig
 
 logger = logging.getLogger(__name__)
@@ -87,7 +94,7 @@ def _gen_token() -> Any:
     return token
 
 
-def _create_s3_client(session: aiobotocore.AioSession, token: Any):  # type: ignore
+def _create_s3_client(session: "AioSession", token: Any):  # type: ignore
     return session.create_client(
         "s3",
         region_name=token.region,
@@ -523,9 +530,7 @@ def _calc_pkg_for_working_dir(
     return (base_pkg, add_pkg, del_pkg, pkgs)
 
 
-async def _upload_pkg(
-    session: aiobotocore.AioSession, token: Any, file_pkg: _Pkg
-) -> bool:
+async def _upload_pkg(session: "AioSession", token: Any, file_pkg: _Pkg) -> bool:
     """Upload the package if it doesn't exist in s3"""
     async with _create_s3_client(session, token) as s3:
         # The content hash is encoded in the path of the package, so we don't
@@ -546,7 +551,7 @@ async def _upload_pkg(
 
 
 async def _upload_file_pkgs(
-    session: aiobotocore.AioSession, token: Any, file_pkgs: List[_Pkg],
+    session: "AioSession", token: Any, file_pkgs: List[_Pkg],
 ) -> None:
     tasks = [_upload_pkg(session, token, pkg) for pkg in file_pkgs]
     _, pending = await asyncio.wait(tasks)
@@ -558,9 +563,7 @@ The following functions are related with package downloading and construction
 """
 
 
-async def _fetch_dir_pkg(
-    session: aiobotocore.AioSession, token: Any, pkg_uri: _PkgURI
-) -> None:
+async def _fetch_dir_pkg(session: "AioSession", token: Any, pkg_uri: _PkgURI) -> None:
     local_path = pkg_uri.local_path()
     with FileLock(str(local_path) + ".lock"):
         if local_path.exists():
@@ -581,7 +584,7 @@ async def _fetch_dir_pkg(
 
 
 async def _fetch_uris(
-    session: aiobotocore.AioSession, token: Any, pkg_uris: List[_PkgURI],
+    session: "AioSession", token: Any, pkg_uris: List[_PkgURI],
 ) -> None:
     tasks = [
         _fetch_dir_pkg(session, token, pkg_uri)
@@ -1111,3 +1114,113 @@ def runtime_env_setup() -> None:
             "Ray 1.5 and 1.6.  Current Ray version: "
             f"{ray.__version__}."
         )
+
+
+def _upload_file_to_google_cloud_storage(file: str, bucket: str, object_name: str):
+    try:
+        from google.cloud import storage
+
+    except Exception:
+        raise click.ClickException(
+            "Could not upload file to Google Storage. Could not import the Google Storage Python API via `from google.cloud import storage`.  Please check your installation or try running `pip install --upgrade google-cloud-storage`."
+        )
+    try:
+        storage_client = storage.Client()
+        bucket_obj = storage_client.bucket(bucket)
+        blob = bucket_obj.blob(object_name)
+        blob.upload_from_filename(file)
+    except Exception as e:
+        raise click.ClickException(
+            f"Could not upload file to Google Cloud Storage. Please check your credentials and ensure that the bucket exists. {e}"
+        )
+
+
+def _upload_file_to_s3(file: str, bucket: str, object_key: str):
+    try:
+        import boto3
+    except Exception:
+        raise click.ClickException(
+            "Could not upload file to S3: Could not import the Amazon S3 Python API via `import boto3`.  Please check your installation or try running `pip install boto3`."
+        )
+    try:
+        s3_client = boto3.client("s3")
+        s3_client.upload_file(file, bucket, object_key)
+    except Exception as e:
+        raise click.ClickException(
+            f"Could not upload file to S3. Check your credentials and that the bucket exists. Original error: {e}"
+        )
+
+
+def _get_remote_storage_object_name(upload_path, upload_filename):
+    # Strip leading slash, otherwise bucket will create a new directory called "/".
+    object_name = os.path.join(urlparse(upload_path).path, upload_filename).lstrip("/")
+    return object_name
+
+
+def _upload_file_to_remote_storage(
+    source_file: str, upload_path: str, upload_filename: str
+):
+    parsed_upload_path = urlparse(upload_path)
+    service = parsed_upload_path.scheme
+    bucket = parsed_upload_path.netloc
+    object_name = _get_remote_storage_object_name(upload_path, upload_filename)
+    if service == "s3":
+        _upload_file_to_s3(source_file, bucket, object_key=object_name)
+    if service == "gs":
+        _upload_file_to_google_cloud_storage(
+            source_file, bucket, object_name=object_name
+        )
+
+    final_uploaded_filepath = os.path.join(upload_path, upload_filename)
+    try:
+        from smart_open import open
+
+        open(final_uploaded_filepath)
+    except Exception as e:
+        raise click.ClickException(
+            f"Could not open uploaded file, maybe something went wrong while uploading: {e}."
+        )
+
+    return final_uploaded_filepath
+
+
+def upload_and_rewrite_working_dir(
+    runtime_env_json: Dict[str, Any],
+    log: BlockLogger = BlockLogger(),
+    upload_file_to_remote_storage_fn: Callable[
+        [str, str, str], str
+    ] = _upload_file_to_remote_storage,
+) -> Dict[str, Any]:
+    """Upload a local working_dir and rewrite the working_dir field with the destination remote URI."""
+    if "working_dir" not in runtime_env_json:
+        return runtime_env_json
+
+    working_dir = runtime_env_json["working_dir"]
+    parsed = urlparse(working_dir)
+    if parsed.scheme:
+        # The working dir is a remote URI already
+        return runtime_env_json
+
+    upload_path = runtime_env_json["upload_path"]
+    with tempfile.TemporaryDirectory() as temp_dir:
+        temp_zip_file_path = os.path.join(
+            temp_dir, "anyscale_generated_working_dir.zip"
+        )
+        zip_directory(
+            working_dir,
+            excludes=runtime_env_json.get("excludes", []),
+            output_path=temp_zip_file_path,
+            # Ray requires remote Zip URIs to consist of a single top-level directory when unzipped.
+            include_parent_dir=True,
+        )
+
+        hash_val = hashlib.md5(Path(temp_zip_file_path).read_bytes()).hexdigest()
+        uploaded_zip_file_name = f"_anyscale_pkg_{hash_val}.zip"
+        final_uploaded_filepath = upload_file_to_remote_storage_fn(
+            temp_zip_file_path, upload_path, uploaded_zip_file_name,
+        )
+
+    final_runtime_env = runtime_env_json.copy()
+    final_runtime_env["working_dir"] = final_uploaded_filepath
+
+    return final_runtime_env
