@@ -42,7 +42,7 @@ add-zsh-hook precmd print_end_output_delimiter
 
 # This is the zsh conversion of the bash jupyter prompt
 # the prompt will appear like: user@host: cwd$
-export PS1="$(whoami)@%M:%~%# "
+export PS1="$(whoami):%~%# "
 
 # Taken from .bashrc initialized by ray
 # >>> conda initialize >>>
