@@ -63,17 +63,21 @@ def test_create_cluster(
         compute_template_id="test_compute_template_id",
         autosuspend_timeout=100,
         allow_public_internet_traffic=None,
+        create_new_cluster=cluster_name is None,
     )
 
 
 @pytest.mark.parametrize("start_required", [True, False])
 @pytest.mark.parametrize("allow_public_internet_traffic", [True, False])
+@pytest.mark.parametrize("create_new_cluster", [True, False])
 def test_start_cluster(
     mock_prepare_cluster_block,
     start_required: bool,
     allow_public_internet_traffic: bool,
+    create_new_cluster: bool,
 ):
     mock_cluster = Mock(id="test_cluster_id")
+    mock_cluster.name = "test_cluster_name"
     mock_prepare_cluster_block._create_or_update_session_data = Mock(
         return_value=(mock_cluster, start_required)
     )
@@ -84,20 +88,22 @@ def test_start_cluster(
         wait_for_session_start=mock_wait_for_session_start,
     ):
         mock_prepare_cluster_block._start_cluster(
-            "test_cluster_name",
             "test_project_id",
+            "test_cluster_name",
             "test_build_id",
             "test_compute_template_id",
             100,
             allow_public_internet_traffic,
+            create_new_cluster,
         )
     mock_prepare_cluster_block._create_or_update_session_data.assert_called_once_with(
-        "test_project_id",
         "test_cluster_name",
+        "test_project_id",
         "test_build_id",
         "test_compute_template_id",
         100,
         bool(allow_public_internet_traffic),
+        create_new_cluster=create_new_cluster,
     )
     mock_prepare_cluster_block._log_cluster_configs.assert_called_once_with(
         mock_cluster, "test_build_id", "test_compute_template_id", ANY
@@ -112,8 +118,8 @@ def test_start_cluster(
             ),
         )
         mock_wait_for_session_start.assert_called_once_with(
-            "test_cluster_name",
             "test_project_id",
+            "test_cluster_name",
             mock_prepare_cluster_block.api_client,
             log=mock_prepare_cluster_block.log,
             block_label=mock_prepare_cluster_block.block_label,
@@ -177,6 +183,62 @@ def test_create_or_update_session_data(
             )
 
 
+def test_create_or_update_session_data_generates_new_cluster_name(
+    mock_prepare_cluster_block,
+):
+    mock_taken_cluster_name = Mock(
+        state="Running", id="test_cluster_id", name="cluster-exists"
+    )
+    mock_cluster = Mock(state="Running", id="test_cluster_id", name="cluster-newgen")
+    mock_prepare_cluster_block.anyscale_api_client.create_cluster.return_value = (
+        mock_cluster
+    )
+
+    mock_get_cluster = Mock(side_effect=[mock_taken_cluster_name, None])
+    mock_prepare_cluster_block._validate_new_cluster_compute_and_env_match_existing_cluster = (
+        Mock()
+    )
+
+    with patch.multiple(
+        "anyscale.connect_utils.prepare_cluster", get_cluster=mock_get_cluster
+    ):
+        mock_prepare_cluster_block._create_or_update_session_data(
+            "cluster-exists",
+            "test_project_id",
+            "test_build_id",
+            "test_compute_template_id",
+            100,
+            None,
+            True,
+        )
+
+    mock_get_cluster.assert_has_calls(
+        [
+            call(
+                mock_prepare_cluster_block.anyscale_api_client,
+                "test_project_id",
+                "cluster-exists",
+            ),
+            call(
+                mock_prepare_cluster_block.anyscale_api_client, "test_project_id", ANY,
+            ),
+        ]
+    )
+
+    mock_prepare_cluster_block.anyscale_api_client.create_cluster.assert_called_once_with(
+        CreateCluster(
+            name=ANY,
+            project_id="test_project_id",
+            cluster_environment_build_id="test_build_id",
+            cluster_compute_id="test_compute_template_id",
+            idle_timeout_minutes=100,
+            allow_public_internet_traffic=None,
+        )
+    )
+    mock_prepare_cluster_block.anyscale_api_client.update_cluster.assert_not_called()
+    mock_prepare_cluster_block._validate_new_cluster_compute_and_env_match_existing_cluster.assert_not_called()
+
+
 @pytest.mark.parametrize("needs_update", [True, False])
 @pytest.mark.parametrize("cluster_exists", [True, False])
 @pytest.mark.parametrize("cluster_name", ["test_cluster_name", None])
