@@ -4,13 +4,16 @@ from unittest.mock import Mock, patch
 from click import ClickException
 import pytest
 
+from anyscale.aws_iam_policies import (
+    AMAZON_ECR_READONLY_ACCESS_POLICY_ARN,
+    AMAZON_S3_FULL_ACCESS_POLICY_ARN,
+)
 from anyscale.client.openapi_client.models import (
     AnyscaleAWSAccount,
     AnyscaleawsaccountResponse,
     Cloud,
     CloudConfig,
     CloudResponse,
-    WriteCloud,
 )
 from anyscale.controllers.cloud_controller import CloudController
 
@@ -68,59 +71,13 @@ def mock_role(document: Optional[Dict[str, Any]] = None) -> Mock:
     return mock_role
 
 
-@pytest.mark.parametrize("region", ["mock-region", None])
-def test_setup_cloud_aioa(region: Optional[str]) -> None:
-
-    expected_region = region or "us-west-2"
-
-    cloud_controller = CloudController()
-    cloud_controller.setup_cloud(
-        is_aioa=True,
-        is_k8s=True,
-        provider="aws",
-        region=region,
-        name="test-aioa",
-        yes=True,
-    )
-    cloud_controller.api_client.create_cloud_api_v2_clouds_post.assert_called_once_with(
-        write_cloud=WriteCloud(
-            provider="AWS",
-            region=expected_region,
-            credentials="",
-            name="test-aioa",
-            is_aioa=True,
-            is_k8s=True,
-        )
-    )
-
-
-def test_setup_cloud_aioa_invalid() -> None:
-
-    cloud_controller = CloudController()
-    with pytest.raises(ClickException):
-        cloud_controller.setup_cloud(
-            is_aioa=True,
-            is_k8s=True,
-            provider="gcp",
-            region="us-west1",
-            name="test-aioa",
-            yes=True,
-        )
-    cloud_controller.api_client.create_cloud_api_v2_clouds_post.assert_not_called()
-
-
 def test_setup_cloud_aws() -> None:
     with patch.object(
         CloudController, "setup_aws", return_value=None
     ) as mock_setup_aws:
         cloud_controller = CloudController()
         cloud_controller.setup_cloud(
-            is_aioa=False,
-            is_k8s=False,
-            provider="aws",
-            region=None,
-            name="test-aws",
-            yes=False,
+            is_k8s=False, provider="aws", region=None, name="test-aws", yes=False,
         )
 
         mock_setup_aws.assert_called_once_with(
@@ -134,12 +91,7 @@ def test_setup_cloud_aws_k8s() -> None:
     ) as mock_setup_aws_k8s_cloud:
         cloud_controller = CloudController()
         cloud_controller.setup_cloud(
-            is_aioa=False,
-            is_k8s=True,
-            provider="aws",
-            region=None,
-            name="test-aws",
-            yes=False,
+            is_k8s=True, provider="aws", region=None, name="test-aws", yes=False,
         )
 
         mock_setup_aws_k8s_cloud.assert_called_once_with(
@@ -155,12 +107,7 @@ def test_setup_cloud_gcp() -> None:
     ):
         cloud_controller = CloudController()
         cloud_controller.setup_cloud(
-            is_aioa=False,
-            is_k8s=True,
-            provider="gcp",
-            region=None,
-            name="test-gcp",
-            yes=True,
+            is_k8s=True, provider="gcp", region=None, name="test-gcp", yes=True,
         )
 
         mock_launch_gcp_cloud_setup.assert_called_once_with(
@@ -175,7 +122,6 @@ def test_setup_cloud_gcp() -> None:
         mock_launch_gcp_cloud_setup.reset_mock()
 
         cloud_controller.setup_cloud(
-            is_aioa=False,
             is_k8s=True,
             provider="gcp",
             region=None,
@@ -219,7 +165,6 @@ def test_setup_cloud_gcp_vpc_peering(
         if vpc_peering_target_project_id is None or vpc_peering_target_vpc_id is None:
             with pytest.raises(ClickException):
                 cloud_controller.setup_cloud(
-                    is_aioa=False,
                     is_k8s=True,
                     provider="gcp",
                     region=None,
@@ -231,7 +176,6 @@ def test_setup_cloud_gcp_vpc_peering(
                 )
         else:
             cloud_controller.setup_cloud(
-                is_aioa=False,
                 is_k8s=True,
                 provider="gcp",
                 region=None,
@@ -257,7 +201,6 @@ def test_setup_cloud_gcp_vpc_peering(
         mock_launch_gcp_cloud_setup.reset_mock()
 
         cloud_controller.setup_cloud(
-            is_aioa=False,
             is_k8s=True,
             provider="gcp",
             region=None,
@@ -313,7 +256,6 @@ def test_setup_cloud_gcp_vpc_peering_validation(
 
         with pytest.raises(ClickException) as e:
             cloud_controller.setup_cloud(
-                is_aioa=False,
                 is_k8s=True,
                 provider="gcp",
                 region=None,
@@ -336,23 +278,17 @@ def test_setup_cloud_gcp_bad_region() -> None:
         # NOTE: GCP regions are [cont]-[local][number], not [cont]-[local]-[number]
         with pytest.raises(ClickException):
             cloud_controller.setup_cloud(
-                is_aioa=False,
-                is_k8s=True,
-                provider="gcp",
-                region="us-west-2",
-                name="test-gcp",
+                is_k8s=True, provider="gcp", region="us-west-2", name="test-gcp",
             )
 
         mock_confirm.assert_called()
 
 
-@pytest.mark.parametrize("is_aioa", [True, False])
 @pytest.mark.parametrize("is_k8s", [True, False])
-def test_setup_cloud_invalid_provider(is_aioa: bool, is_k8s: bool) -> None:
+def test_setup_cloud_invalid_provider(is_k8s: bool) -> None:
     cloud_controller = CloudController()
     with pytest.raises(ClickException):
         cloud_controller.setup_cloud(
-            is_aioa=is_aioa,
             is_k8s=is_k8s,
             provider="azure",
             region="azure-west-1",
@@ -468,11 +404,11 @@ def test_setup_aws_ray_role(roles: List[Optional[Mock]]) -> None:
         mock_iam.create_role.assert_called_once()
 
     # Assert we actually attached the base policies
-    roles[-1].attach_policy.assert_called()
-
-    # And let it created our PassRole policy as well
-    roles[-1].Policy.assert_called_with("PassRoleToSelf")
-    roles[-1].Policy().put.assert_called()
+    roles[-1].attach_policy.assert_any_call(PolicyArn=AMAZON_S3_FULL_ACCESS_POLICY_ARN)
+    roles[-1].attach_policy.assert_any_call(
+        PolicyArn=AMAZON_ECR_READONLY_ACCESS_POLICY_ARN
+    )
+    assert 2 == roles[-1].attach_policy.call_count
 
 
 def test_update_cloud_config_by_name(cloud_test_data: Cloud) -> None:
