@@ -1,5 +1,4 @@
 import json
-import os
 from typing import List
 from unittest.mock import Mock, mock_open, patch
 
@@ -8,7 +7,6 @@ import pytest
 
 from anyscale.controllers.cluster_controller import ClusterController
 from anyscale.sdk.anyscale_client import StartClusterOptions
-from anyscale.util import PROJECT_NAME_ENV_VAR
 
 
 @pytest.mark.parametrize(
@@ -68,7 +66,8 @@ def test_get_project_id_and_cluster_name_given_cluster_id(mock_auth_api_client):
         return_value="mock_cluster_name1"
     )
     with patch.multiple(
-        "anyscale.controllers.cluster_controller", validate_project_id=Mock(),
+        "anyscale.controllers.cluster_controller",
+        get_and_validate_project_id=Mock(return_value="mock_project_id1"),
     ):
         assert cluster_controller._get_project_id_and_cluster_name(
             cluster_id=None,
@@ -81,11 +80,9 @@ def test_get_project_id_and_cluster_name_given_cluster_id(mock_auth_api_client):
     cluster_controller._get_or_generate_cluster_name = Mock(  # type: ignore
         return_value="mock_cluster_name1"
     )
-    mock_get_proj_id_from_name = Mock(return_value="mock_project_id1")
     with patch.multiple(
         "anyscale.controllers.cluster_controller",
-        validate_project_id=Mock(),
-        get_proj_id_from_name=mock_get_proj_id_from_name,
+        get_and_validate_project_id=Mock(return_value="mock_project_id1"),
     ):
         assert cluster_controller._get_project_id_and_cluster_name(
             cluster_id=None,
@@ -94,38 +91,6 @@ def test_get_project_id_and_cluster_name_given_cluster_id(mock_auth_api_client):
             project_name="mock_project_name1",
         ) == ("mock_project_id1", "mock_cluster_name1")
 
-    # Test using default project
-    cluster_controller._get_or_generate_cluster_name = Mock(  # type: ignore
-        return_value="mock_cluster_name2"
-    )
-    cluster_controller.anyscale_api_client.get_default_project = Mock(
-        return_value=Mock(result=Mock(id="mock_project_id2"))
-    )
-    with patch.multiple(
-        "anyscale.controllers.cluster_controller",
-        load_project_or_throw=Mock(side_effect=click.ClickException("")),
-    ):
-        assert cluster_controller._get_project_id_and_cluster_name(
-            cluster_id=None, project_id=None, cluster_name=None, project_name=None
-        ) == ("mock_project_id2", "mock_cluster_name2")
-
-    # Test getting project_name from env var
-    cluster_controller._get_or_generate_cluster_name = Mock(  # type: ignore
-        return_value="mock_cluster_name3"
-    )
-    mock_get_proj_id_from_name = Mock(return_value="mock_project_id3")
-    with patch.multiple(
-        "anyscale.controllers.cluster_controller",
-        validate_project_id=Mock(),
-        get_proj_id_from_name=mock_get_proj_id_from_name,
-    ), patch.dict(os.environ, {PROJECT_NAME_ENV_VAR: "project_from_env"}):
-        assert cluster_controller._get_project_id_and_cluster_name(
-            cluster_id=None, project_id=None, cluster_name=None, project_name=None,
-        ) == ("mock_project_id3", "mock_cluster_name3")
-        mock_get_proj_id_from_name.assert_called_once_with(
-            "project_from_env", cluster_controller.api_client
-        )
-
 
 def test_get_cluster_env_and_build(mock_auth_api_client):
     cluster_controller = ClusterController()
@@ -209,6 +174,7 @@ def test_start(mock_auth_api_client, needs_start: bool) -> None:
     cluster_controller._create_or_update_cluster_data = Mock(  # type: ignore
         return_value=(Mock(id="cluster_id"), needs_start)
     )
+    mock_user_service_access = "private"
 
     with patch.multiple(
         "anyscale.controllers.cluster_controller", wait_for_session_start=Mock(),
@@ -217,12 +183,16 @@ def test_start(mock_auth_api_client, needs_start: bool) -> None:
             cluster_name=None,
             cluster_id="cluster_id",
             cluster_env_name="cluster_env_name",
+            docker=None,
+            python_version=None,
+            ray_version=None,
             cluster_compute_name="cluster_compute_name",
             cluster_compute_file=None,
             cloud_name=None,
             idle_timeout=None,
             project_id=None,
             project_name=None,
+            user_service_access=mock_user_service_access,
         )
 
     cluster_controller._get_project_id_and_cluster_name.assert_called_once_with(
@@ -242,6 +212,7 @@ def test_start(mock_auth_api_client, needs_start: bool) -> None:
         True,
         True,
         None,
+        mock_user_service_access,
     )
 
     if needs_start:
