@@ -3,21 +3,39 @@ import os
 from pathlib import Path
 import tempfile
 import time
-from typing import Any, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Optional, Tuple, Type
 from unittest.mock import Mock, mock_open, patch
 
 import click
 import pytest
 import yaml
 
-from anyscale.cli_logger import BlockLogger
 from anyscale.client.openapi_client import CreateProductionJob, ProductionJobConfig
-from anyscale.controllers.job_controller import JobConfig, JobController, LogProvider
+from anyscale.controllers.job_controller import (
+    _working_dir_is_remote_uri,
+    JobConfig,
+    JobController,
+    LogProvider,
+    LogsLogger,
+    MiniJobRun,
+)
 from anyscale.util import PROJECT_NAME_ENV_VAR
 
 
 CONDA_DICT = {"dependencies": ["pip", {"pip": ["pip-install-test==0.5"]}]}
 PIP_LIST = ["requests==1.0.0", "pip-install-test"]
+ENV_VARS_DICT = {"TEST_ENV_VAR": "test_value"}
+
+
+class FakeLogger(LogsLogger):
+    logs: List[str] = []
+
+    def open_block(self, *args, **kwargs,) -> None:
+        pass
+
+    def log(self, msg: str):
+        self.logs.append(msg)
+        print(msg)
 
 
 @pytest.fixture
@@ -285,6 +303,7 @@ def test_terminate_job(
 
 class TestValidateConda:
     def test_validate_conda_str(self, test_directory, patch_jobs_anyscale_api_client):
+        """Tests the conda field is allowed to be a str. It represents an existing env name."""
         jc = JobConfig(
             entrypoint="ls",
             build_id="123",
@@ -294,6 +313,7 @@ class TestValidateConda:
         assert jc.runtime_env["conda"] == "env_name"
 
     def test_validate_conda_invalid_path(self, patch_jobs_anyscale_api_client):
+        """If a path to a YAML file is given, it should error if the path does not exist."""
         with pytest.raises(click.ClickException):
             JobConfig(
                 entrypoint="ls",
@@ -323,6 +343,7 @@ class TestValidateConda:
     def test_validate_conda_invalid_file(
         self, test_directory, absolute_path, patch_jobs_anyscale_api_client
     ):
+        """We should error if a .yml file with invalid YAML format is specified."""
         _, _, _, bad_conda_file = test_directory
 
         if absolute_path:
@@ -348,6 +369,7 @@ class TestValidateConda:
 
 class TestValidatePip:
     def test_validate_pip_invalid_path(self, patch_jobs_anyscale_api_client):
+        """If a path to a .txt file is given, it should error if the path does not exist."""
         with pytest.raises(click.ClickException):
             JobConfig(
                 entrypoint="ls",
@@ -383,8 +405,94 @@ class TestValidatePip:
         assert jc.runtime_env["pip"] == PIP_LIST
 
 
+class TestValidateEnvVars:
+    def test_validate_env_vars_valid_dict(self, patch_jobs_anyscale_api_client):
+        jc = JobConfig(
+            entrypoint="ls",
+            build_id="123",
+            compute_config_id="test",
+            runtime_env={"env_vars": ENV_VARS_DICT},
+        )
+        assert jc.runtime_env["env_vars"] == ENV_VARS_DICT
+
+    def test_validate_env_vars_invalid_dict(self, patch_jobs_anyscale_api_client):
+        """Error if env_vars is not a dict."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"env_vars": "not_a_dict"},
+            )
+
+    def test_validate_env_vars_not_dict_str(self, patch_jobs_anyscale_api_client):
+        """Error if env_vars is not a Dict[str, str]."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"env_vars": {"key", 123}},
+            )
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"env_vars": {"key": "value", 123: "value2"}},
+            )
+
+
+class TestValidateUploadPath:
+    def test_reject_upload_path_without_local_dir(self, patch_jobs_anyscale_api_client):
+        """If upload_path is specified, a local working_dir must also be specified."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"upload_path": "s3://some_bucket"},
+            )
+
+    def test_reject_non_s3_or_gs_upload_path(self, patch_jobs_anyscale_api_client):
+        """If upload_path is specified, it must be an s3 or gs path."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"upload_path": "file://some_file"},
+            )
+
+    def test_accept_s3_upload_path(self, patch_jobs_anyscale_api_client):
+        jc = JobConfig(
+            entrypoint="ls",
+            build_id="123",
+            compute_config_id="test",
+            runtime_env={"working_dir": "./", "upload_path": "s3://some_bucket"},
+        )
+        assert jc.runtime_env["upload_path"] == "s3://some_bucket"
+
+    def test_accept_gs_upload_path(self, patch_jobs_anyscale_api_client):
+        jc = JobConfig(
+            entrypoint="ls",
+            build_id="123",
+            compute_config_id="test",
+            runtime_env={"working_dir": "./", "upload_path": "gs://some_bucket"},
+        )
+        assert jc.runtime_env["upload_path"] == "gs://some_bucket"
+
+
 class TestValidateWorkingDir:
-    def test_reject_local_dir(self, patch_jobs_anyscale_api_client):
+    def test_working_dir_is_remote_uri(self):
+        assert _working_dir_is_remote_uri("s3://some_bucket")
+        assert _working_dir_is_remote_uri("s3://some_bucket/some/path")
+        assert _working_dir_is_remote_uri("fake://some_bucket/some/path/")
+        assert not _working_dir_is_remote_uri("/some/path")
+        assert not _working_dir_is_remote_uri("some/path/")
+
+    def test_reject_local_dir_without_upload_path(self, patch_jobs_anyscale_api_client):
+        """If a local working_dir is specified, upload_path must also be specified."""
         with pytest.raises(click.ClickException):
             JobConfig(
                 entrypoint="ls",
@@ -401,6 +509,44 @@ class TestValidateWorkingDir:
                 runtime_env={"working_dir": "/tmp/dir"},
             )
 
+    def test_reject_remote_uri_with_upload_path(self, patch_jobs_anyscale_api_client):
+        """If a working_dir is specified as a remote URI, upload_path cannot be specified."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={"working_dir": "s3://test", "upload_path": "s3://another"},
+            )
+
+    def test_accept_local_dir_with_upload_path(self, patch_jobs_anyscale_api_client):
+        JobConfig(
+            entrypoint="ls",
+            build_id="123",
+            compute_config_id="test",
+            runtime_env={"working_dir": ".", "upload_path": "s3://fake"},
+        )
+
+        JobConfig(
+            entrypoint="ls",
+            build_id="123",
+            compute_config_id="test",
+            runtime_env={"working_dir": ".", "upload_path": "gs://fake/dir"},
+        )
+
+    def test_reject_nonexistent_local_dir(self, patch_jobs_anyscale_api_client):
+        """If a local working_dir is specified, it must exist."""
+        with pytest.raises(click.ClickException):
+            JobConfig(
+                entrypoint="ls",
+                build_id="123",
+                compute_config_id="test",
+                runtime_env={
+                    "working_dir": "/does/not/exist",
+                    "upload_path": "gs://fake",
+                },
+            )
+
     def test_accept_uri(self, patch_jobs_anyscale_api_client):
         jc = JobConfig(
             entrypoint="ls",
@@ -413,6 +559,7 @@ class TestValidateWorkingDir:
 
 class TestValidatePyModules:
     def test_reject_local_dir(self, patch_jobs_anyscale_api_client):
+        """Local directories are only supported using working_dir, not py_modules."""
         with pytest.raises(click.ClickException):
             JobConfig(
                 entrypoint="ls",
@@ -479,14 +626,16 @@ def test_validate_project_id_field(
                 assert job_config.project_id == project_id
 
 
-@pytest.mark.parametrize("mode", ["empty", "single", "paginate", "follow"])
-def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
+def fake_log_provider(mode: str) -> Type[LogProvider]:
     class FakeLogProvider(LogProvider):
         def __init__(self, *args, **kwargs) -> None:
             self.constructed_at = int(time.time() * 1e9)
             self.page = -1
             self.query_count = 0
 
+        def close(self):
+            pass
+
         def query(
             self, start_timestamp_ns: int, end_timestamp_ns: int
         ) -> List[Tuple[int, str]]:
@@ -514,16 +663,64 @@ def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
                     empty_list,
                     [(start_timestamp_ns, "finally")],
                 ][self.page]
+            if mode == "flush_success":
+                self.page += 1
+                return (
+                    [
+                        # simulate no log
+                        empty_list,
+                        empty_list,
+                        [(start_timestamp_ns, "finally")],
+                        empty_list,
+                        # Simulate 30 steps of empty logs
+                    ]
+                    + [empty_list for _ in range(30)]
+                )[self.page]
+            if mode == "wait_for_running":
+                self.page += 1
+                return (
+                    [
+                        # simulate no log
+                        empty_list,
+                        empty_list,
+                        [(start_timestamp_ns, "finally")],
+                        empty_list,
+                        # Simulate 30 steps of empty logs
+                    ]
+                    + [empty_list for _ in range(30)]
+                )[self.page]
 
             raise Exception("unreachable")
 
-    class FakeLogger(BlockLogger):
-        def open_block(
-            self, _block_label: str, _block_title: Optional[str] = None
-        ) -> None:
-            pass
+    return FakeLogProvider
 
+
+@pytest.mark.parametrize(
+    "mode",
+    ["empty", "single", "paginate", "follow", "flush_success", "wait_for_running"],
+)
+def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
     job_controller = JobController(log=FakeLogger())
+    fake_log_provider_cls = fake_log_provider(mode)
+
+    state = {"flush_success": "SUCCESS", "wait_for_running": "PENDING"}.get(
+        mode, "RUNNING"
+    )
+
+    job_controller._get_formatted_latest_job_run = Mock(  # type: ignore
+        return_value=MiniJobRun(
+            last_job_run_id="mock_job_id", job_state=state, error=None
+        )
+    )
+
+    def reset(*args, **kwargs):
+        job_controller._get_formatted_latest_job_run = Mock(  # type: ignore
+            return_value=MiniJobRun(
+                last_job_run_id="mock_job_id", job_state="SUCCESS", error=None
+            )
+        )
+
+    job_controller._wait_for_job_running = Mock(side_effect=reset)  # type: ignore
     job_controller.anyscale_api_client.get_job_logs_query_info = Mock(
         return_value=Mock(
             result=Mock(
@@ -538,21 +735,55 @@ def test_job_logs(mock_auth_api_client, capsys, mode) -> None:
         job_controller.logs(
             job_id="mock_job_id",
             job_name=None,
-            should_follow=mode == "follow",
-            log_provider_cls=FakeLogProvider,
+            should_follow=mode in ["follow", "flush_success", "wait_for_running"],
+            log_provider_cls=fake_log_provider_cls,
         )
 
     if mode == "empty":
         do_logs()
         assert capsys.readouterr().out.strip() == ""
-    if mode == "single":
+    elif mode == "single":
         do_logs()
         assert capsys.readouterr().out.strip() == "start\nend"
-    if mode == "paginate":
+    elif mode == "paginate":
         do_logs()
         assert capsys.readouterr().out.strip() == "entry-1\nentry-2"
-    if mode == "follow":
+    elif mode == "follow":
         with pytest.raises(IndexError):  # controller should continue to query
             do_logs()
         out = capsys.readouterr().out.strip()
         assert out == "finally"
+    elif mode == "flush_success":
+        do_logs()
+        out = capsys.readouterr().out.strip()
+        assert out == "finally"
+    elif mode == "wait_for_running":
+        with pytest.raises(IndexError):  # controller should continue to query
+            do_logs()
+        assert job_controller._wait_for_job_running.call_count == 1
+        out = capsys.readouterr().out.strip()
+        assert out == "finally"
+    else:
+        raise Exception("unreachable")
+
+
+def test_wait_for_job_running(mock_auth_api_client) -> None:
+    job_controller = JobController(log=FakeLogger())
+    i = 0
+
+    def get_latest_job_run(*args, **kwargs):
+        print("Called get_latest_job_run")
+        nonlocal i
+        i += 1
+        ret = Mock()
+        status = "RUNNING"
+        if i < 3:
+            status = "PENDING"
+        elif i < 5:
+            status = "AWAITING_CLUSTER_START"
+        ret.state.current_state = status
+        return ret
+
+    job_controller._get_latest_job_run = Mock(side_effect=get_latest_job_run)  # type: ignore
+    job_controller._wait_for_job_running("fake_job_id")
+    assert i == 5
