@@ -813,6 +813,7 @@ def test_ssh_session(
     mock_ray_nodes = Mock(return_value=mock_node_data)
 
     mock_exists = Mock(return_value=on_head_node)
+    mock_get_and_validate_project_id = Mock()
 
     with patch.multiple(
         "anyscale.controllers.session_controller",
@@ -820,6 +821,7 @@ def test_ssh_session(
         get_project_id=mock_get_project_id,
         get_cluster_config=mock_get_cluster_config,
         get_project_session=mock_get_project_session,
+        get_and_validate_project_id=mock_get_and_validate_project_id,
     ), patch.multiple("subprocess", run=Mock()), patch(
         "builtins.open", mock_open(read_data=str(mock_cluster_config))
     ), patch.multiple(
@@ -840,6 +842,7 @@ def test_ssh_session(
                     worker_node_id=worker_node_id,
                     worker_node_ip=worker_node_ip,
                     force_head_node=force_head_node,
+                    project_id=None,
                 )
             return
         else:
@@ -849,6 +852,7 @@ def test_ssh_session(
                 worker_node_id=worker_node_id,
                 worker_node_ip=worker_node_ip,
                 force_head_node=force_head_node,
+                project_id=None,
             )
 
     if on_head_node and worker_node_id is not None:
@@ -865,12 +869,11 @@ def test_ssh_session(
         session_controller.api_client.get_session_head_ip_api_v2_sessions_session_id_head_ip_get.assert_called_once_with(
             session_test_data.id,
         )
-        mock_load_project_or_throw.assert_called_once_with()
-        mock_get_project_id.assert_called_once_with("/some/directory")
-        mock_get_cluster_config.assert_called_once_with(
-            session_test_data.name,
-            session_controller.api_client,
-            disable_project_sync=True,
+        mock_get_and_validate_project_id.assert_called_once_with(
+            project_id=None,
+            project_name=None,
+            api_client=session_controller.api_client,
+            anyscale_api_client=session_controller.anyscale_api_client,
         )
 
 
