@@ -8,7 +8,7 @@ from anyscale.auth_proxy import authorize, index, make_auth_proxy_app
 
 
 async def test_auth_token_redirect_serve():
-    make_auth_proxy_app("mock_auth_token")
+    make_auth_proxy_app("mock_auth_token", None)
     mock_request = Mock(
         query={"token": "mock_auth_token", "redirect_to": None, "auth_only": None},
         headers={"Host": "serve-session-id.i.anyscaleuserdata-staging.com"},
@@ -39,7 +39,7 @@ async def test_auth_token_redirect_serve():
 )
 @pytest.mark.parametrize("auth_only", [True, None])
 async def test_auth_token_redirect_services(service: str, auth_only: Optional[bool]):
-    make_auth_proxy_app("mock_auth_token")
+    make_auth_proxy_app("mock_auth_token", None)
     mock_request = Mock(
         query={
             "token": "mock_auth_token",
@@ -76,6 +76,27 @@ async def test_auth_token_redirect_services(service: str, auth_only: Optional[bo
         expected_cookies["anyscale-token"]["secure"] = True
     assert str(resp.cookies) == str(expected_cookies)
 
+    # Test redirect for user service urls
+    make_auth_proxy_app("mock_auth_token", "mock_user_service_token")
+    mock_request = Mock(
+        query={
+            "token": "mock_user_service_token",
+            "redirect_to": "docs",
+            "auth_only": None,
+        },
+        headers={"Host": "serve-session-id.i.anyscaleuserdata-staging.com"},
+    )
+
+    resp = await index(mock_request)
+    assert resp.status == 302
+    assert (
+        resp.location == "https://serve-session-id.i.anyscaleuserdata-staging.com/docs"
+    )
+    expected_cookies_user_service: SimpleCookie = SimpleCookie()
+    expected_cookies_user_service["anyscale-token"] = "mock_user_service_token"
+    expected_cookies_user_service["anyscale-token"]["path"] = "/"
+    assert str(resp.cookies) == str(expected_cookies_user_service)
+
 
 @pytest.mark.parametrize(
     "mock_request",
@@ -85,9 +106,14 @@ async def test_auth_token_redirect_services(service: str, auth_only: Optional[bo
         Mock(headers={"Authorization": "bearer mock_auth_token"}),
     ],
 )
-async def test_authorize_success(mock_request: Mock):
-    mock_request.config_dict = {"auth_token": "mock_auth_token"}
-    make_auth_proxy_app("mock_auth_token")
+@pytest.mark.parametrize("use_token", ["access_token", "user_service_token"])
+async def test_authorize_success(mock_request: Mock, use_token: str):
+    mock_request.config_dict = {
+        "auth_token": "mock_auth_token",
+        "user_service_token": "mock_auth_token",
+    }
+    mock_request.query = {"use_token": use_token}
+    make_auth_proxy_app("mock_auth_token", "mock_auth_token")
 
     resp = await authorize(mock_request)
     assert resp.status == 200
@@ -102,9 +128,14 @@ async def test_authorize_success(mock_request: Mock):
         Mock(),
     ],
 )
-async def test_authorize_failure(mock_request: Mock):
-    mock_request.config_dict = {"auth_token": "mock_auth_token"}
-    make_auth_proxy_app("mock_auth_token")
+@pytest.mark.parametrize("use_token", ["access_token", "user_service_token"])
+async def test_authorize_failure(mock_request: Mock, use_token: str):
+    mock_request.config_dict = {
+        "auth_token": "mock_auth_token",
+        "user_service_token": "mock_auth_token",
+    }
+    mock_request.query = {"use_token": use_token}
+    make_auth_proxy_app("mock_auth_token", "mock_auth_token")
 
     resp = await authorize(mock_request)
     assert resp.status == 401
