@@ -631,7 +631,7 @@ def test_new_proj_connect_params(
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
         job_config=builder._job_config,
-        allow_multiple_clients=False,
+        allow_multiple_clients=True,
         initial_scale=builder._initial_scale,
         in_shell=builder._in_shell,
         run_mode=builder._run_mode,
@@ -1014,6 +1014,7 @@ def test_set_runtime_env_in_job_config(
     mock_cluster = connect_instance._anyscale_sdk.search_sessions.return_value.results[
         0
     ]
+    mock_cluster.user_service_url = "mock_user_service_url"
     mock_get_prepare_cluster_block.return_value.cluster_name = mock_cluster.name
     mock_get_project_block.return_value.project_dir = proj_dir
 
@@ -1028,7 +1029,7 @@ def test_set_runtime_env_in_job_config(
                 "/tmp/.anyscale.yaml",
                 "/tmp/session-default.yaml",
             ],
-            "env_vars": {"RAY_SERVE_ROOT_URL": "https://serve-session-0.userdata.com"},
+            "env_vars": {"RAY_SERVE_ROOT_URL": mock_cluster.user_service_url},
         }
     )
     mock_get_prepare_cluster_block.assert_called_once_with(
@@ -1052,7 +1053,7 @@ def test_set_runtime_env_in_job_config(
     mock_get_start_interactive_session_block.assert_called_once_with(
         cluster=mock_cluster,
         job_config=connect_instance._job_config,
-        allow_multiple_clients=False,
+        allow_multiple_clients=True,
         initial_scale=connect_instance._initial_scale,
         in_shell=connect_instance._in_shell,
         run_mode=connect_instance._run_mode,
